// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package database

import (
	"fmt"
	"gorm.io/gorm/logger"
	"log"

	"gorm.io/driver/mysql"
	"gorm.io/gorm"
	"gorm.io/plugin/dbresolver"
)

var db *gorm.DB


// Setup 建立连接
func Setup(user, passwd, addr, dbname, charset string, maxIdle, maxConn int, logMode bool) *gorm.DB {
	// 组装参数
	args := fmt.Sprintf("%s:%s@tcp(%s)/%s?charset=%s&parseTime=True&loc=Local",
		user, passwd, addr, dbname, charset)
	// 打开新连接
	var err error
	config := &gorm.Config{}
	if logMode {
		config.Logger=logger.Default.LogMode(logger.Info)
	}
	db, err = gorm.Open(mysql.Open(args), config)
	if err != nil {
		log.Fatal("open mysql connection failed. err: ", err)
	}
	sqlDB, err := db.DB()
	if err != nil {
		log.Fatal("open mysql sqlDB failed. err: ", err)
	}
	// 其他设置
	//db.LogMode(logMode)

	sqlDB.SetMaxIdleConns(maxIdle)
	sqlDB.SetMaxOpenConns(maxConn)

	return db
}

// 读写分离使用，备用，暂未测过
func ClusterSetup(masters []string, slaves []string, maxIdle, maxConn int) *gorm.DB {
	var err error
	if len(masters) == 0 {
		panic("no masters")
	}
	db, err = gorm.Open(mysql.Open(masters[0]), &gorm.Config{})
	if err != nil {
		log.Fatal("open mysql connection failed. err: ", err)
	}
	sqlDB, err := db.DB()
	if err != nil {
		log.Fatal("open mysql sqlDB failed. err: ", err)
	}

	sources := []gorm.Dialector{}
	replicas := []gorm.Dialector{}
	for _, v := range masters {
		sources = append(sources, mysql.Open(v))
	}
	for _, v := range slaves {
		replicas = append(replicas, mysql.Open(v))
	}
	dbResolverCfg := dbresolver.Config{
		Sources:  sources,
		Replicas: replicas,
		Policy:   dbresolver.RandomPolicy{}}
	readWritePlugin := dbresolver.Register(dbResolverCfg)
	db.Use(readWritePlugin)
	// 其他设置
	//db.LogMode(logMode)
	sqlDB.SetMaxIdleConns(maxIdle)
	sqlDB.SetMaxOpenConns(maxConn)

	return db
}

// DB 获取连接
func DB() *gorm.DB {
	return db
}

// Close 关闭连接
func Close() {
	if db != nil {
		sqlDb, err := db.DB()
		if err == nil {
			sqlDb.Close()
		}
	}
}
